//
//  DMActivationController.h
//  DevMateActivation
//
//  Created by Dmytro Tretiakov on 12/24/12.
//  Copyright (c) 2012 MacPaw Inc. All rights reserved.
//

#import <Cocoa/Cocoa.h>

#if !__has_feature(objc_arc)
#define DMAutorelease(v) ([v autorelease])
#define DMRetain(v) ([v retain])
#define DMRelease(v) ([v release])
#else // -fobjc-arc
#define DMAutorelease(v) (v)
#define DMRetain(v) (v)
#define DMRelease(v) (v)
#endif

#ifndef NS_ENUM
#define NS_ENUM(_type, _name) _type _name; enum
#endif

//! Enum of standard activation step implemented in current framework
typedef NS_ENUM(NSInteger, DMActivationStandardStep)
{
    DMActivationWelcomeStep = 0,
    DMActivationActivationStep,
    DMActivationSuccessStep,
    DMActivationFailureStep,
};
typedef NSInteger DMActivationStep;

//! Enum of activation modes
typedef NS_ENUM(NSInteger, DMActivationMode)
{
    DMActivationIndependentMode = 0,
    DMActivationChildMode,
    DMActivationModalMode,
    DMActivationSheetMode,
    DMActivationFloatingMode
};

typedef void (^DMCompletionHandler)(BOOL success);

//! Forward declarations
@protocol DMActivationControllerDelegate;
@protocol DMActivator;
@class DMStepController;

//! Main controller for activation
@interface DMActivationController : NSWindowController
{
@private
    NSMutableDictionary *_stepsMap;
    NSMutableDictionary *_stepsStateContainer;
    
    id<DMActivator> activator;
    __unsafe_unretained id<DMActivationControllerDelegate> delegate;
    DMActivationStep currentStep;
    DMStepController *currentStepController;
    DMCompletionHandler completionHandler;
    DMActivationStep nextPerformStep;
    DMActivationMode activationMode;
    
    __weak NSView *containerView;
}

//! Returns shared controller instance
+ (id)sharedController;

//! Initializes new controller with default window
//  To use own XIB for controller use -initWithWindowNibName: method instead
+ (id)defaultController;

@property (nonatomic, strong) id<DMActivator> activator;
@property (nonatomic, unsafe_unretained) id<DMActivationControllerDelegate> delegate;

@property (nonatomic, assign, readonly) DMActivationStep currentStep;
@property (nonatomic, strong, readonly) DMStepController *currentStepController;

//! All saved steps states including current step.
@property (nonatomic, readonly) NSDictionary *allStepsStates;

//! Method that shows activation window and starts ativation process
//  @params
//      activationMode - activation mode
//      initialActivationInfo - initial info that will be used for activation, can be nil
//      completionHandler - completion handler
- (void)runActivationWindowInMode:(DMActivationMode)activationMode
            initialActivationInfo:(NSDictionary *)initialActivationInfo
            withCompletionHandler:(DMCompletionHandler)completionHandler;

//! Use this mehod to register own steps with own controllers or
//  to re-register standard steps with own controller or XIB for standard controller
//  @params
//      controllerClassName - controller class for registered step. Use nil to remove step controller info.
//                            Must be subclass of DMStepController
//      controllerNibName - XIB in main bundle for controller.
//      step - activation step (custom or DMActivationStandardStep) to register new controller for it
- (void)registerStepController:(NSString *)controllerClassName
                   withNibName:(NSString *)controllerNibName
             forActivationStep:(DMActivationStep)step;

//! Outlet for view container where steps views will be places into
@property (nonatomic, weak, readonly) IBOutlet NSView *containerView;

//! Method to change activation step view. Uses in step controllers.
- (void)performActivationStepWithStep:(DMActivationStep)step;

//! Ends activation dialog and calls success/failure completion handler.
//  Uses in step controllers.
- (void)endActivationWithSuccess:(BOOL)success;

@end

@protocol DMActivationControllerDelegate <NSObject>
@optional
    
//! Must be implemented for DMActivationChildMode and DMActivationSheetMode activation mode
//  will be centered in parent window for other modes
- (NSWindow *)activationController:(DMActivationController *)controller parentWindowForActivationMode:(DMActivationMode)mode;

//! Can be implemented to adjust standard steps. This is the last chance to register controller for returned/proposed activation step
- (DMActivationStep)activationController:(DMActivationController *)controller activationStepForProposedStep:(DMActivationStep)proposedStep;

//! Will be called before changing currentStep and currentStepController properties with new ones.
- (void)activationControllerWillChangeActivationStep:(DMActivationController *)controller;

//! Will be called after currentStep and currentStepController properties were changed to new ones
//  and step view was moved to controller container view
- (void)activationControllerDidChangeActivationStep:(DMActivationController *)controller;

//! Delegate should implement this method to correct activation info. Otherwise proposedInfo will be used for activation
- (NSDictionary *)activationController:(DMActivationController *)controller activationInfoWithProposedInfo:(NSDictionary *)proposedInfo;

@end
